#Web VPython 3.2

from vpython import curve, box, vector, radians, color, scene, rate, sphere, canvas

title = """Symmetry planes in a cubic crystal

&#x2022; Original <a href="https://lectdemo.github.io/virtual/24_crystal_planes.html">24_crystal_planes.py</a> by Ruth Chabay Spring 2001
&#x2022; Refactored by <a href="https://www.hendrikse.name/">Zeger Hendrikse</a>
&#x2022; &lt;c&gt; &rarr; toggle wireframe cube
&#x2022; &lt;s&gt; &rarr; screenshot
&#x2022; &lt;v&gt; &rarr; verbose output
&#x2022; &lt;space&gt; &rarr; toggle background color
&#x2022; &lt;mouse click&gt; &rarr; toggle between different symmetry planes

"""

animation = canvas(background=color.gray(0.075), forard=vector(1, -0.8, -2), title = title)

class WireFrame:
    def __init__(self, s=5., box_color=vector(0, 1, 1)):
        pts = [vector(-s, -s, -s), vector(-s, -s, s), vector(-s, s, s),
               vector(-s, s, -s), vector(-s, -s, -s), vector(s, -s, -s),
               vector(s, s, -s), vector(-s, s, -s), vector(s, s, -s),
               vector(s, s, s), vector(-s, s, s), vector(s, s, s),
               vector(s, -s, s), vector(-s, -s, s), vector(s, -s, s), vector(s, -s, -s)]
        self._cube = curve(color=box_color, radius=0.05, pos=pts)

    def set_visibility_to(self, true_or_false):
        self._cube.visible = true_or_false

    def toggle_visibility(self):
        self._cube.visible = not self._cube.visible


class SymmetryPlanes:
    def __init__(self):
        self._planes = [box(pos=vector(0, 0, 0), size=vector(6, 0.01, 6), color=color.gray(0.8), visible=False) for _ in
                        range(3)]
        self._current_plane = -1
        self._planes[1].rotate(axis=vector(0, 0, 1), angle=radians(45))
        self._planes[2].rotate(axis=vector(0, 0, 1), angle=radians(90))

    def toggle_visibility(self):
        self._current_plane = 0 if self._current_plane + 1 > 3 else self._current_plane + 1
        for index in range(len(self._planes)):
            self._planes[index].visible = (self._current_plane == index)
        cube.set_visibility_to(self._current_plane == 3)


class Crystal:
    def __init__(self):
        for x in range(-2, 3, 2):
            for z in range(-2, 3, 2):
                for y in range(-2, 3, 2):
                    sphere(pos=vector(x, y, z), radius=0.3, color=vector(1, 0, 1))
                    
        for x in range(-1, 3, 2):
            for z in range(-1, 3, 2):
                for y in range(-1, 3, 2):
                    sphere(pos=vector(x, y, z), radius=0.3, color=vector(0, 1, 1))


cube = WireFrame(s=1, box_color=vector(.8, .8, .8))
crystal = Crystal()
symmetry_planes = SymmetryPlanes()


def toggle_background():
    animation.background = color.gray(0.075) if animation.background is color.white else color.white

def toggle_symmetry_planes():
    symmetry_planes.toggle_visibility()

def on_key_press(event):
    if event.key == "c":
        cube.toggle_visibility()
    if event.key == " ":
        toggle_background()
    if event.key == 's':
        animation.capture("crystal_planes")
    if event.key == 'v':
        print("scene.center=" + str(animation.center))
        print("scene.forward=" + str(animation.forward))
        print("scene.range=" + str(animation.range))


animation.bind("keydown", on_key_press)
animation.bind("click", toggle_symmetry_planes)

while 1:
    rate(10)
